#include<xc.h>

#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config BOREN = ON       // Brown-out Reset Enable bit (BOR enabled)
#pragma config LVP = OFF        // Low-Voltage (Single-Supply) In-Circuit Serial Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit (Data EEPROM code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off; all program memory may be written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)


#define _XTAL_FREQ 20000000 //We are running on 20MHz crystal 

//Define the 7-seg control
#define s1 RC0
#define s2 RC1
const unsigned char SEGMENT_MAP[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7C,0x7F,0x6F};

/*Set the current value of date and time below*/
int sec = 00;
int min = 55;
int hour = 10;
int date = 06;
int month = 05;
int year = 18;
/*Time and Date Set*/


//----------------------------------------------------------------------------------I2C --------------------------//
void I2C_Initialize(const unsigned long feq_K) //Begin IIC as master
{
  TRISC3 = 1;  TRISC4 = 1;  //Set SDA and SCL pins as input pins
  
  SSPCON  = 0b00101000;    //pg84/234 
  SSPCON2 = 0b00000000;    //pg85/234
  
  SSPADD = (_XTAL_FREQ/(4*feq_K*100))-1; //Setting Clock Speed pg99/234 //// WHY ITS 100 SHULD BE 1000
  SSPSTAT = 0b00000000;    //pg83/234
}

void I2C_Hold()
{
    while (   (SSPCON2 & 0b00011111)    ||    (SSPSTAT & 0b00000100)   ) ; //check the bis on registers to make sure the IIC is not in progress
}

void I2C_Begin()
{
  I2C_Hold();  //Hold the program is I2C is busy  
  SEN = 1;     //Begin IIC pg85/234
}



void I2C_End()
{
  I2C_Hold(); //Hold the program is I2C is busy  
  PEN = 1;    //End IIC pg85/234
}

void I2C_Write(unsigned data)
{
  I2C_Hold(); //Hold the program is I2C is busy 
  SSPBUF = data;         //pg82/234
}

unsigned short I2C_Read(unsigned short ack)
{
  unsigned short incoming;
  I2C_Hold();
  RCEN = 1;
  
  I2C_Hold();
  incoming = SSPBUF;      //get the data saved in SSPBUF
  
  I2C_Hold();
  ACKDT = (ack)?0:1;    //check if ack bit received  
  ACKEN = 1;          //pg 85/234
  
  return incoming;
}


//****************************************************************************** Functions for RTC module *******/

int  BCD_2_DEC(int to_convert)
{
   return (to_convert >> 4) * 10 + (to_convert & 0x0F); 
    //return ((to_convert/16 * 10) + (to_convert % 16));
}

int DEC_2_BCD (int to_convert)
{
   return ((to_convert / 10) << 4) + (to_convert % 10);
    //return ((to_convert/10)<<4) + (to_convert % 10);
}

void Set_Time_Date()
{
   I2C_Begin();       
   I2C_Write(0xD0); 
   I2C_Write(0);  
   I2C_Write(DEC_2_BCD(sec)); //update sec
   I2C_Write(DEC_2_BCD(min)); //update min
   I2C_Write(DEC_2_BCD(hour)); //update hour
   I2C_Write(1); //ignore updating day
   I2C_Write(DEC_2_BCD(date)); //update date
   I2C_Write(DEC_2_BCD(month)); //update month
   I2C_Write(DEC_2_BCD(year)); //update year
   I2C_End();
}

void Update_Current_Date_Time()
{
   //START to Read
   I2C_Begin();       
   I2C_Write(0xD0); 
   I2C_Write(0);    
   I2C_End(); 
   
  //READ
   I2C_Begin();
   I2C_Write(0xD1);                              // Initialize data read
   sec = BCD_2_DEC(I2C_Read(1));    
   min = BCD_2_DEC(I2C_Read(1));   // Read sec from register 
   hour = BCD_2_DEC(I2C_Read(1));  
   I2C_Read(1);
   date = BCD_2_DEC(I2C_Read(1));  
   month = BCD_2_DEC(I2C_Read(1));  
   year = BCD_2_DEC(I2C_Read(1));  
   I2C_End(); 
    
  //END Reading  
    I2C_Begin();
    I2C_Write(0xD1);                              // Initialize data read
    I2C_Read(1);    
    I2C_End(); 

}


int main()
{ 
    TRISC = 0xFC; //RC0, RC1 output pin
    TRISD = 0x00; //Make Port D pins as outptu for SEGMENTS 
    PORTD = 0x00;

    I2C_Initialize(100); //Initialize I2C Master with 100KHz clock    
    Set_Time_Date(); //set time and date on the RTC module
    

while(1)
  {
 
    Update_Current_Date_Time(); //Read the current date and time from RTC module
    
    //Split the into char to display on lcd
     unsigned int sec_0 = (sec%10);
     unsigned int sec_1 = (sec/10);    
     
    
    //Display the Time sec
     
     PORTD = SEGMENT_MAP[sec_0]; s2 = 1; 
     __delay_ms(10); s2 = 0;
     
     PORTD = SEGMENT_MAP[sec_1]; s1 = 1; 
     __delay_ms(10); s1 = 0;

     
  }

    return 0;
}